package database;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

public class DatabaseConnectionCreator {
	
	private Connection databaseConnection;
	
	/**
	 * Creates connection to the database with given credentials.
	 * @param url url of the database
	 * @param user user name
	 * @param password password for given user
	 * @throws SQLException
	 */
	public DatabaseConnectionCreator(String url, String user, String password) throws SQLException {
		databaseConnection = DriverManager.getConnection(url, user, password);
	}
	
	/**
	 * Creates connection to the database with given credentials in config file.
	 * @param configFile file with credentials
	 * @throws IOException
	 * @throws SQLException
	 */
	public DatabaseConnectionCreator(String configFile) throws IOException, SQLException {
		File file = new File(configFile);
		FileReader fileReader = new FileReader(file);
		BufferedReader bufferedReader = new BufferedReader(fileReader);
		String url = bufferedReader.readLine();
		String user = bufferedReader.readLine();
		String password = bufferedReader.readLine();
		databaseConnection = DriverManager.getConnection(url, user, password);
		bufferedReader.close();
	}
	
	/**
	 * Returns connection to the database
	 * @return connection to the database
	 */
	public Connection getConnection() {
		return databaseConnection;
	}

}
