package dump;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import main.Constants;

public class DumpReader {
		
	private BufferedReader bufferedReader;
	
	private FileReader fileReader;

	/**
	 * Creates buffered reader over given file.
	 * @param fileName file to read
	 * @throws IOException
	 */
	public DumpReader(String fileName) throws IOException {
		File file = new File(fileName);
		this.fileReader = new FileReader(file);
		this.bufferedReader = new BufferedReader(fileReader);
	}
	
	/**
	 * It returns list of lines of next found page.
	 * @return list of lines of next found page
	 * @throws IOException
	 */
	public List<String> getNextPage() throws IOException {
		List<String> linesOfPage = null;
		boolean pageFound = false;
		boolean pageCompleted = false;
		while (!pageCompleted) {
			String line = bufferedReader.readLine();
			
			if (pageFound) {
				linesOfPage.add(line);
			}
			
			if (line.contains(Constants.PAGE_START)) {
				pageFound = true;
				linesOfPage = new ArrayList<String>();
			}
			else if (line.contains(Constants.PAGE_END)) {
				pageCompleted = true;
			}			
		}
		
		if (linesOfPage == null) {
			return null;
		}
		
		return linesOfPage;
	}
	
}
