package main;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.sql.SQLException;

import database.DatabaseConnectionCreator;
import database.DatabaseInserter;
import dump.DumpReader;

/**
 * Program takes 5 parameters: database ip, database name, database user name, database user's password, path to Wikipedia dump file.
 * This order must be followed. If no paramaters are given, then it try to load config file.
 * @author Karel Šilhavý
 *
 */
public class Main {
	
	public static void main(String[] args) {
		DatabaseConnectionCreator databaseHandler = null;
		DumpReader dumpReader = null;
				
		if (args.length == 5) {
			try {
				String url = "jdbc:mysql://" + args[0] + "/" + args[1] + "?useUnicode=true&useJDBCCompliantTimezoneShift=true&useLegacyDatetimeCode=false&serverTimezone=UTC&useSSL=false";
				databaseHandler = new DatabaseConnectionCreator(url, args[2], args[3]);
			} catch (SQLException e) {
				System.err.println("Database with entered credentials is unreachable.");
				System.exit(1);
			}
			
			try {
				dumpReader = new DumpReader(args[4]);
			} catch (IOException e) {
				System.err.println("Error! Dump file missing or can not be opened!");
				System.exit(1);
			}
			DatabaseInserter databaseInserter = new DatabaseInserter(databaseHandler, dumpReader);
			databaseInserter.start();
		}
		else if (new File(Constants.CONFIG_FILE_NAME).exists()) {
			File file = new File(Constants.CONFIG_FILE_NAME);
			FileReader fileReader = null;
			try {
				fileReader = new FileReader(file);
				BufferedReader bufferedReader = new BufferedReader(fileReader);
				
				String ip = bufferedReader.readLine();
				String dbName = bufferedReader.readLine();
				String user = bufferedReader.readLine();
				String password = bufferedReader.readLine();
				
				String url = "jdbc:mysql://" + ip + "/" + dbName + "?useUnicode=true&useJDBCCompliantTimezoneShift=true&useLegacyDatetimeCode=false&serverTimezone=UTC&useSSL=false";
				try {
					databaseHandler = new DatabaseConnectionCreator(url, user, password);
				} catch (SQLException e) {
					System.err.println("Database with entered credentials is unreachable.");
					System.exit(1);
				}
				
				String dumpFilePath = bufferedReader.readLine();
				
				try {
					dumpReader = new DumpReader(dumpFilePath);
				} catch (IOException e) {
					System.err.println("Error! Dump file missing or can not be opened!");
					System.exit(1);;
				}
				
				bufferedReader.close();				
			} catch (IOException e) {
				e.printStackTrace();
			}
			DatabaseInserter databaseInserter = new DatabaseInserter(databaseHandler, dumpReader);
			databaseInserter.start();
		}
		else {
			System.out.println("Program takes 5 parameters: database ip, database name, database user name, database user's password, path to Wikipedia dump file.This order must be followed. If no paramaters are given, then it try to load config file.");			
		}
	}
	
}
