package datahandlers;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DatabaseHandler {
	
	private Connection databaseConnection;
	
	public DatabaseHandler(Connection databaseConnection) {
		this.databaseConnection = databaseConnection;
	}
		
	public Connection getConnection() {
		return databaseConnection;
	}
	
	public InfoboxEntry getInfobox(String infoboxName) throws SQLException {
		PreparedStatement preparedStatement1 = databaseConnection.prepareStatement("SELECT * FROM pages where pageName=?");
		preparedStatement1.setString(1, infoboxName);
		
		ResultSet rs = preparedStatement1.executeQuery();
		
		int id = -1;
		if (rs.next()) {
			id = rs.getInt("id");
		}
		
		PreparedStatement preparedStatement2 = databaseConnection.prepareStatement("SELECT * FROM infoboxes where pageId=?");
		preparedStatement2.setInt(1, id);
		
		rs = preparedStatement2.executeQuery();
		
		InfoboxEntry infoboxEntry = null;
		
		while (rs.next()) {
			String type = rs.getString("infoboxType");
			LinkedHashMap<String, List<InfoboxLink>> content = parseInfoboxContent(rs.getString("infoboxContent"));
			
			List<KeyWithProperty> keys = new ArrayList<KeyWithProperty>();
			List<List<InfoboxLink>> links = new ArrayList<List<InfoboxLink>>();
			
			for (Map.Entry<String, List<InfoboxLink>> entry: content.entrySet()) {
				keys.add(new KeyWithProperty (entry.getKey(), "")); ///// !!!!!!!!!!!!!!!! ///////
				links.add(entry.getValue());
			}
			infoboxEntry = new InfoboxEntry(infoboxName, type, keys, links);
		}
		
		if (infoboxEntry == null) {
			return getRedirect(infoboxName);
		}
		
		return infoboxEntry;		
	}

	public LinkedHashMap<String, List<InfoboxLink>> parseInfoboxContent(String infoboxContent) {
		LinkedHashMap<String, List<InfoboxLink>> pairs = new LinkedHashMap<String, List<InfoboxLink>>();
		
		List<String> infoboxLines = new ArrayList<String>(Arrays.asList(infoboxContent.split("[\\r\\n]+")));
		for (int i = 0; i < infoboxLines.size(); i++) {
			String line = infoboxLines.get(i).trim();
			if (line.length() == 0) {
				continue;
			}
			if (line.charAt(0) == '|') {
				int indexOfKeyEnds = line.indexOf('=');
				if (indexOfKeyEnds == -1) {
					continue;
				}
				String key = line.substring(1, indexOfKeyEnds).trim();
				List<InfoboxLink> values = new ArrayList<InfoboxLink>();
				
				//handle multiple values
				String lineBeforeObejct = "";
				for (int j = i; j < infoboxLines.size(); j++) {
					line = infoboxLines.get(j).trim();
					if (line.contains("{{")) {
						int indexOfObjectStart = line.indexOf("{{");
						
						//in case of list iterating
						try {
							lineBeforeObejct = line.substring(indexOfKeyEnds + 1, indexOfObjectStart).trim();							
						} catch (StringIndexOutOfBoundsException e) {
							lineBeforeObejct = "";
						}
						//end of that case
						
						line = line.substring(indexOfObjectStart);
						values.addAll(parseValues(line));
					}
					else {
						values.addAll(parseValues(line));
					}
					
					if (j + 1 < infoboxLines.size()) {
						line = infoboxLines.get(j+1).trim();
						if (line.length() == 0) {
							continue;
						}
						if (line.charAt(0) == '|') {
							break;
						}
					}
				}
				//handle single value
				if (lineBeforeObejct.length() > 0) {
					values.addAll(parseValues(lineBeforeObejct));
				}
				if (values.size() == 0) {
					line = infoboxLines.get(i).trim();
					if (line.substring(indexOfKeyEnds + 1).trim().length() > 0) {
						values.add(new InfoboxLink(line.substring(indexOfKeyEnds + 1).trim(), null));
						pairs.put(key, values);						
					}
				}
				else {
					pairs.put(key, values);
				}
			}
		}
		
		return pairs;
	}
	
	private List<InfoboxLink> parseValues(String rawValues) {
		List<InfoboxLink> values = new ArrayList<InfoboxLink>();
		Pattern regex = Pattern.compile("\\[\\[(.+?)\\]\\]"); //between [[ ]]
		Matcher regexMatcher = regex.matcher(rawValues);

		while (regexMatcher.find()) {
			String found = regexMatcher.group(1);
			if (found.contains("|")) {
				String[] titledLink = found.split("\\|");
				values.add(new InfoboxLink(titledLink[1], titledLink[0]));
			} 
			else {
				values.add(new InfoboxLink(found, found));
			}
		}
		return values;
	}
		
	public InfoboxEntry getRedirect(String oldName) {
		PreparedStatement preparedStatement;
		try {
			String query = ("SELECT * FROM redirect where oldName=?");
			preparedStatement = databaseConnection.prepareStatement(query);
			preparedStatement.setString(1, oldName);
			
			ResultSet rs = preparedStatement.executeQuery();
			
			while (rs.next()) {
				String newName = rs.getString("newName");

				if (newName.equals(oldName)) {
					continue;
				}
				InfoboxEntry infoboxFromRedirect = getInfobox(newName);
				if (infoboxFromRedirect == null) {
					continue;
				}

				return infoboxFromRedirect;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	public String getInfoboxValueForKey(String infoboxName, String key) {
		PreparedStatement preparedStatement1;
		String infoboxContent = null;
		try {
			preparedStatement1 = databaseConnection.prepareStatement("SELECT * FROM wikipedia where infoboxName=?");
			preparedStatement1.setString(1, infoboxName);
			
			ResultSet rs = preparedStatement1.executeQuery();
			
			if (rs.next()) {
				infoboxContent = rs.getString("infoboxContent");
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		int keyIndexStart = infoboxContent.indexOf(key);
		int keyWithValueIndexEnd = infoboxContent.length() - 1;
		for (int i = keyIndexStart; i < infoboxContent.length(); i++) {
			if (infoboxContent.charAt(i) == '|') {
				keyWithValueIndexEnd = i;
				break;
			}
		}
		int valueIndexStart = infoboxContent.substring(keyIndexStart, keyWithValueIndexEnd).indexOf("=");
		
		return infoboxContent.substring(valueIndexStart + 1, valueIndexStart).trim();			
	}

}
