package datahandlers;

import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import main.Constants;

public class GraphVertex {
	
	private String name;

	private String fullHtmlPageString;
	
	private DateTime birthDate;
	
	private DateTime deathDate;
	
	private DateTimeFormatter dateTimeFormatter;
	
	public GraphVertex() {
		this.dateTimeFormatter = DateTimeFormat.forPattern(Constants.DATE_TIME_PATTERN);
	}
	
	public GraphVertex(String name) {
		this.name = name;
		this.dateTimeFormatter = DateTimeFormat.forPattern(Constants.DATE_TIME_PATTERN);
	}

	public GraphVertex(String name, String fullHtmlPageString) {
		this.name = name;
		this.fullHtmlPageString = fullHtmlPageString;
		this.dateTimeFormatter = DateTimeFormat.forPattern(Constants.DATE_TIME_PATTERN);
	}
	
	public GraphVertex(String name, DateTime birthDate, DateTime deathDate) {
		this.name = name;
		this.birthDate = birthDate;
		this.deathDate = deathDate;
		this.dateTimeFormatter = DateTimeFormat.forPattern(Constants.DATE_TIME_PATTERN);
	}
	
	public GraphVertex(String name, String fullHtmlPageString, DateTime birthDate, DateTime deathDate) {
		this.name = name;
		this.fullHtmlPageString = fullHtmlPageString;
		this.birthDate = birthDate;
		this.deathDate = deathDate;
		this.dateTimeFormatter = DateTimeFormat.forPattern(Constants.DATE_TIME_PATTERN);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public String getFullHtmlPageString() {
		return fullHtmlPageString;
	}
	
	public DateTime getBirthDate() {
		return birthDate;
	}
	
	public String getBirthDateString() {
		if (birthDate == null) {
			return "";
		}
		else {
			return dateTimeFormatter.print(birthDate);			
		}
	}

	public void setBirthDate(DateTime birthDate) {
		this.birthDate = birthDate;
	}
	
	public void setBirthDateString(String birthDate) {
		this.birthDate = dateTimeFormatter.parseDateTime(birthDate);
	}

	public DateTime getDeathDate() {
		return deathDate;
	}
	
	public String getDeathDateString() {
		if (deathDate == null) {
			return "";
		}
		else {
			return dateTimeFormatter.print(deathDate);			
		}
	}

	public void setDeathDate(DateTime deathDate) {
		this.deathDate = deathDate;
	}
	
	public void setDeathDateString(String deathDate) {
		this.deathDate = dateTimeFormatter.parseDateTime(deathDate);
	}

	public void setFullHtmlPageString(String fullHtmlPageString) {
		this.fullHtmlPageString = fullHtmlPageString;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		GraphVertex other = (GraphVertex) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "GraphVertex [name=" + name + ", birthDate=" + birthDate + ", deathDate=" + deathDate + "]";
	}

}
