package datahandlers;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.json.Json;
import javax.json.JsonArrayBuilder;
import javax.json.JsonObjectBuilder;
import javax.json.JsonWriter;
import javax.json.JsonWriterFactory;
import javax.json.stream.JsonGenerator;

import javafx.util.Pair;

public class TimelineExporter {
	
	private String HEADER = "define([],function(){return{";
	
	private static String TAIL = "};});";
	
	private List<Pair<Integer, String>> pairs; 
	
	private File exportFile;
	
	public TimelineExporter(File exportFile) {
		this.pairs = new ArrayList<Pair<Integer, String>>();
		this.exportFile = exportFile;
	}
	
	public void export(JgraphtDataHandler jgraphtDataHandler) {
		JsonObjectBuilder nodesJson = nodesXML(jgraphtDataHandler.getAllVertices());
		JsonObjectBuilder edgesJson = edgesXML(jgraphtDataHandler, jgraphtDataHandler.getAllEdges());		
		
		Map<String, Object> properties = new HashMap<>(1);
		properties.put(JsonGenerator.PRETTY_PRINTING, true);
		
		JsonWriterFactory writerFactory = Json.createWriterFactory(properties);
		StringWriter stringWriter = new StringWriter();
		JsonWriter jsonWriter = writerFactory.createWriter(stringWriter);

		jsonWriter.writeObject(nodesJson.build());
		jsonWriter.close();
		
		String nodesJsonString = stringWriter.toString();
		nodesJsonString = nodesJsonString.substring(2, nodesJsonString.length()-2);
		
		stringWriter = new StringWriter();
		jsonWriter = writerFactory.createWriter(stringWriter);
		
		jsonWriter.writeObject(edgesJson.build());
		jsonWriter.close();
		
		String edgesJsonString = stringWriter.toString();
		edgesJsonString = edgesJsonString.substring(2, edgesJsonString.length()-2);
		
		String output = HEADER + System.lineSeparator() + nodesJsonString + System.lineSeparator() + ", " + System.lineSeparator()
						+ edgesJsonString + System.lineSeparator() + TAIL;
		try {
			PrintWriter writer = new PrintWriter(exportFile.getAbsolutePath(), "UTF-8");
			writer.write(output);
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
		
	private JsonObjectBuilder nodesXML(Set<GraphVertex> allVertices) {
		JsonObjectBuilder nodesWrapper = Json.createObjectBuilder();
		JsonArrayBuilder nodesArray = Json.createArrayBuilder();
		
		Iterator<GraphVertex> nodesIterator = allVertices.iterator();
		for (int i = 0; nodesIterator.hasNext(); i++) {
			JsonObjectBuilder nodeJSON = Json.createObjectBuilder();
			GraphVertex node = nodesIterator.next();
			
			nodeJSON.add("id", i);
			nodeJSON.add("stereotype", "person");
			nodeJSON.add("name", node.getName());
			nodeJSON.add("description", node.getName());
			
			nodeJSON.add("begin", node.getBirthDateString());				
			nodeJSON.add("end", node.getDeathDateString());
			
			JsonObjectBuilder propertiesArray = Json.createObjectBuilder();
			propertiesArray.add("startPrecision", "day");
			propertiesArray.add("endPrecision", "day");
			
			nodeJSON.add("properties", propertiesArray);
			
			nodesArray.add(nodeJSON);
			pairs.add(new Pair<Integer, String>(new Integer(i), node.getName()));
		}
		
		nodesWrapper.add("nodes", nodesArray);
		
		return nodesWrapper;
	}
	
	private JsonObjectBuilder edgesXML(JgraphtDataHandler jgraphtDataHandler, Set<RelationshipEdge> allEdges) {
		JsonObjectBuilder edgesWrapper = Json.createObjectBuilder();
		JsonArrayBuilder edgesArray = Json.createArrayBuilder();
		
		Iterator<RelationshipEdge> edgesIterator = allEdges.iterator();
		for (int i = 0; edgesIterator.hasNext(); i++) {
			JsonObjectBuilder edgeJSON = Json.createObjectBuilder();
			RelationshipEdge edge = edgesIterator.next();
			
			edgeJSON.add("id", i);
			edgeJSON.add("stereotype", "relationship");
			edgeJSON.add("from", getNodeNumber(jgraphtDataHandler.getEdgeSource(edge).getName()));
			edgeJSON.add("to", getNodeNumber(jgraphtDataHandler.getEdgeTarget(edge).getName()));
			edgeJSON.add("name", edge.getLabel());
			
			edgesArray.add(edgeJSON);
		}
		
		edgesWrapper.add("edges", edgesArray);
		
		return edgesWrapper;
	}
	
	private int getNodeNumber(String nodeName) {
		for (Pair<Integer, String> pair: pairs) {
			if (pair.getValue().equals(nodeName)) {
				return pair.getKey().intValue();
			}
		}
		return -1;
	}

}
