package fxmlcontrollers;

import java.io.File;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Queue;
import java.util.Set;

import datahandlers.DatabaseConnectionCreator;
import datahandlers.DatabaseHandler;
import datahandlers.GraphVertex;
import datahandlers.InfoboxEntry;
import datahandlers.InfoboxLink;
import datahandlers.JgraphtDataHandler;
import datahandlers.KeyWithProperty;
import datahandlers.RelationshipEdge;
import datahandlers.TableRowInfoboxData;
import datahandlers.TimelineExporter;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ListView;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.CellEditEvent;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.Pane;
import javafx.scene.web.WebView;
import javafx.stage.FileChooser;
import javafx.stage.FileChooser.ExtensionFilter;
import javafx.stage.Stage;
import main.Constants;

public class MainWindowController {
	
    @FXML
    private BorderPane mainBorderPane;

    @FXML
    private TextField searchField;

    @FXML
    private ListView<String> suggestedLinkingPropertiesListView;

    @FXML
    private WebView webView;

    @FXML
    private ListView<String> edgesListView;
    
    @FXML
    private TableView<TableRowInfoboxData> tableOverview;
    
    @FXML
    private TableColumn<TableRowInfoboxData, String> birthDateColumn;

    @FXML
    private TableColumn<TableRowInfoboxData, String> deathDateColumn;
    
    @FXML
    private TableColumn<TableRowInfoboxData, String> nameColumn;

    @FXML
    private TableColumn<TableRowInfoboxData, String> realitonsWithColumn;

    @FXML
    private TableColumn<TableRowInfoboxData, String> relationsColumn;
    
    @FXML
    private TextField customFromTextField;
    
    @FXML
    private TextField customToTextField;

    @FXML
    private TextField customPropertyTextField;
    		
	private DatabaseHandler databaseHandler;
	
	private JgraphtDataHandler jgraphtDataHandler;
		
	private GraphVertex lastSelectedVertex;
		
	public MainWindowController() throws SQLException {
		DatabaseConnectionCreator databaseConnection;
		try {
			databaseConnection = new DatabaseConnectionCreator(Constants.CONFIG_FILE_NAME);
			databaseHandler = new DatabaseHandler(databaseConnection.getConnection());
		} catch (IOException e) {
    		Alert alert = new Alert(AlertType.ERROR);
    		alert.setTitle("Config file not found!");
    		alert.setContentText("Config file not found!");
    		alert.showAndWait();
    		Platform.exit();
		}
    	jgraphtDataHandler = new JgraphtDataHandler(databaseHandler);
	}
	
    @FXML
    void initialize() {
    	//table's cells init
    	birthDateColumn.setCellValueFactory(new PropertyValueFactory<TableRowInfoboxData, String>(birthDateColumn.getId()));
    	birthDateColumn.setCellFactory(TextFieldTableCell.forTableColumn());
    	deathDateColumn.setCellValueFactory(new PropertyValueFactory<TableRowInfoboxData, String>(deathDateColumn.getId()));
    	deathDateColumn.setCellFactory(TextFieldTableCell.forTableColumn());
    	nameColumn.setCellValueFactory(new PropertyValueFactory<TableRowInfoboxData, String>(nameColumn.getId()));
    	realitonsWithColumn.setCellValueFactory(new PropertyValueFactory<TableRowInfoboxData, String>(realitonsWithColumn.getId()));
    	relationsColumn.setCellValueFactory(new PropertyValueFactory<TableRowInfoboxData, String>(relationsColumn.getId()));
    	
    	//table init
    	tableOverview.getSelectionModel().selectedItemProperty().addListener((obs, oldSelection, newSelection) -> {
    		if (newSelection != null) {
    			lastSelectedVertex = jgraphtDataHandler.getVertexByName(newSelection.getNameColumn());
    			updateWebView();
    			updateTableSelection();
    			
    			searchField.setText(newSelection.getNameColumn());
    			/// "on search clicked" ///
    	    	String searchedText = searchField.getText();
    	    	InfoboxEntry infoboxEntry = null;
				try {
					infoboxEntry = databaseHandler.getInfobox(searchedText);
				} catch (Exception e) {
					e.printStackTrace();
				}
				
    	    	if (infoboxEntry == null) {
    	    		return;
    	    	}
    	    	List<KeyWithProperty> keys = infoboxEntry.getKeys();
    			
    	    	suggestedLinkingPropertiesListView.getItems().clear();
    	    	for (KeyWithProperty key: keys) {
    	    		if (key.getProperty().length() == 0) {
    	    			suggestedLinkingPropertiesListView.getItems().add(key.getKey());    			
    	    		}
    	    		else {
    	    			suggestedLinkingPropertiesListView.getItems().add(key.getKey() + " [" + key.getProperty() + "]");
    	    		}
    	    	}
    	    	
    	    	suggestedLinkingPropertiesListView.getItems().clear();
    	    	for (KeyWithProperty key: keys) {
    	    		if (key.getProperty().length() == 0) {
    	    			suggestedLinkingPropertiesListView.getItems().add(key.getKey());    			
    	    		}
    	    		else {
    	    			suggestedLinkingPropertiesListView.getItems().add(key.getKey() + " [" + key.getProperty() + "]");
    	    		}
    	    	}
    		}
    	});
    	
    	//edge list init
		edgesListView.getSelectionModel().selectedItemProperty().addListener((obs, oldSelection, newSelection) -> {
			lastSelectedVertex = jgraphtDataHandler.getVertexByName(newSelection);
			updateWebView();
			updateTableSelection();
		});
		
		
		jgraphtDataHandler.saveCurrentData();
    }

    

    
    @FXML
    void onDeleteEdgeClicked(ActionEvent event) {
		String edgeToDelete = edgesListView.getSelectionModel().getSelectedItem();
		edgesListView.getItems().remove(edgeToDelete);
		jgraphtDataHandler.deleteEdge(edgeToDelete);
		updateTableView();
		jgraphtDataHandler.saveCurrentData();
    }

    @FXML
    void onResetClicked(ActionEvent event) {
    	webView.getEngine().loadContent("");
    	jgraphtDataHandler.clearGraph();
    	updateEdgesListView();
    	updateTableView();
    	jgraphtDataHandler.saveCurrentData();
    }
    
    @FXML
    void onExportClicked(ActionEvent event) {
    	Set<GraphVertex> allVertices = jgraphtDataHandler.getAllVertices();
    	
    	if (allVertices.size() == 0) {
    		Alert alert = new Alert(AlertType.INFORMATION);
    		alert.setTitle("Nothing to export");
    		alert.setContentText("There are no data to export.");
    		alert.showAndWait();
    		return;
    	}
    	
    	for (GraphVertex vertex: allVertices) {
    		if (vertex.getBirthDate() == null || vertex.getDeathDate() == null) {
        		Alert alert = new Alert(AlertType.WARNING);
        		alert.setTitle("Missing date");
        		alert.setContentText("Entry without filled date found. Fill the dates first or remove entries without them.");
        		alert.showAndWait();
        		return;
    		}
    	}
    	
    	FileChooser fileChooser = new FileChooser();
    	
    	Scene scene = new Scene(new Pane());
    	Stage stage = new Stage();
    	stage.setScene(scene);
    	
    	fileChooser.getExtensionFilters().add(new ExtensionFilter("All files", "*.*"));
    	fileChooser.setInitialFileName("data.js");
    	File selectedFile = fileChooser.showSaveDialog(stage);
    	
    	TimelineExporter timelineExporter = new TimelineExporter(selectedFile);
    	timelineExporter.export(jgraphtDataHandler);
    }
    
    @FXML
    void onUndoClicked(ActionEvent event) {
    	jgraphtDataHandler.loadDataBefore();
    	updateEdgesListView();
    	updateTableView();
    }
    
    @FXML
    void onRedoClicked(ActionEvent event) {
    	jgraphtDataHandler.loadDataAfter();
    	updateEdgesListView();
    	updateTableView();    	
    }
    
    @FXML
    void onSearchClicked(ActionEvent event) throws SQLException {
    	String searchedText = searchField.getText();
    	InfoboxEntry infoboxEntry = databaseHandler.getInfobox(searchedText);
    	if (infoboxEntry == null) {
    		Alert alert = new Alert(AlertType.WARNING);
    		alert.setTitle("Entered infobox not found!");
    		alert.setContentText("Page you are searching for has been not found. Enter exactly same name as it is on Wikipedia.");
    		alert.showAndWait();
    		return;
    	}
    	List<KeyWithProperty> keys = infoboxEntry.getKeys();
    	suggestedLinkingPropertiesListView.getItems().clear();
    	for (KeyWithProperty key: keys) {
    		if (key.getProperty().length() == 0) {
    			suggestedLinkingPropertiesListView.getItems().add(key.getKey());    			
    		}
    		else {
    			suggestedLinkingPropertiesListView.getItems().add(key.getKey() + " [" + key.getProperty() + "]");
    		}
    	}
    }
    
    @FXML
    void onApplyOnAllClicked(ActionEvent event) throws SQLException {
    	String selectedProperty = suggestedLinkingPropertiesListView.getSelectionModel().getSelectedItem();
    	
    	Set<GraphVertex> currentVertices = new HashSet<GraphVertex>(jgraphtDataHandler.getAllVertices());
    	
	    for (GraphVertex vertex: currentVertices) {
	    	if (selectedProperty != null) {
	    		fillGraph(vertex.getName(), selectedProperty);
	    	}
	    }
	    jgraphtDataHandler.saveCurrentData();
    }
    
    @FXML
    void onLinkFromClicked(ActionEvent event) throws SQLException {
    	String selectedProperty = suggestedLinkingPropertiesListView.getSelectionModel().getSelectedItem();
    	if (selectedProperty != null) {
    		fillGraph(searchField.getText(), selectedProperty);
    	}
    	jgraphtDataHandler.saveCurrentData();
    }
        
    @FXML
    void onUseSelectedClicked(ActionEvent event) {
    	if (lastSelectedVertex != null) {
    		String selection = (String) webView.getEngine().executeScript("window.getSelection().toString()");
    		customFromTextField.setText(lastSelectedVertex.getName());
    		customPropertyTextField.setText(selection);
    	}
    }
    
    @FXML
    void onUseSelectedPropertyClicked(ActionEvent event) {
    	customPropertyTextField.setText(getSelectedTextFromWebviewAndSetcustomFromTextField());
    }
    
    @FXML
    void onUseSelectedToClicked(ActionEvent event) {
    	customToTextField.setText(getSelectedTextFromWebviewAndSetcustomFromTextField());
    }
    
    @FXML
    void onHardLinkClicked(ActionEvent event) throws SQLException {
    	String linkFrom = customFromTextField.getText();
    	String linkTo = customToTextField.getText();
    	String linkingProperty = customPropertyTextField.getText();
    	
    	jgraphtDataHandler.insertInfoboxIntoGraph(linkFrom);
    	jgraphtDataHandler.insertInfoboxIntoGraph(linkTo);
    	
		GraphVertex linkFromGraphVertex = jgraphtDataHandler.getVertexByName(linkFrom);
		GraphVertex linkToGraphVertex = jgraphtDataHandler.getVertexByName(linkTo);
		
    	jgraphtDataHandler.insertEdge(linkFromGraphVertex, linkToGraphVertex, linkingProperty + "[" + linkFromGraphVertex + "-" + linkToGraphVertex + "]");
    	
	    updateEdgesListView();
	    updateTableView();
	    jgraphtDataHandler.saveCurrentData();
    }
        
    @FXML
    void onDeleteFromTableClicked(ActionEvent event) {
    	for (GraphVertex vertex: jgraphtDataHandler.getAllVertices()) {
    		if (vertex.getName().equals(lastSelectedVertex.getName())) {
    			jgraphtDataHandler.deleteVertex(vertex.getName());
    			break;
    		}
    	}
    	updateTableView();
    	updateEdgesListView();
    	jgraphtDataHandler.saveCurrentData();
    }
    
    @FXML
    void birthDateOnEditCommit(CellEditEvent<TableRowInfoboxData, String> editEvent) {
    	try {
    		jgraphtDataHandler.getVertexByName(editEvent.getRowValue().getNameColumn()).setBirthDateString(editEvent.getNewValue());;    		
    	} catch (IllegalArgumentException e) {    		
    		if (editEvent.getNewValue().equals(Constants.IF_DATE_IS_NULL)) {
    			jgraphtDataHandler.getVertexByName(editEvent.getRowValue().getNameColumn()).setBirthDate(null);
    		}
    		else {
	    		invalidDateFormatAlert();
	    		tableOverview.getItems().set(editEvent.getTablePosition().getRow(), tableOverview.getItems().get(editEvent.getTablePosition().getRow())); //refresh old value to not display new one
	    		return;
    		}
    	}
		updateTableView();
		jgraphtDataHandler.saveCurrentData();
    }
    
    @FXML
    void deathDateOnEditCommit(CellEditEvent<TableRowInfoboxData, String> editEvent) {
    	try {
    		jgraphtDataHandler.getVertexByName(editEvent.getRowValue().getNameColumn()).setDeathDateString(editEvent.getNewValue());;    		
    	} catch (IllegalArgumentException e) {    		
    		if (editEvent.getNewValue().equals(Constants.IF_DATE_IS_NULL)) {
    			jgraphtDataHandler.getVertexByName(editEvent.getRowValue().getNameColumn()).setDeathDate(null);
    		}
    		else {
	    		invalidDateFormatAlert();
	    		tableOverview.getItems().set(editEvent.getTablePosition().getRow(), tableOverview.getItems().get(editEvent.getTablePosition().getRow())); //refresh old value to not display new one
	    		return;
    		}
    	}
		updateTableView();
    	jgraphtDataHandler.saveCurrentData();
    }
	
	private void invalidDateFormatAlert() {
		Alert alert = new Alert(AlertType.WARNING);
		alert.setTitle("Invalid date format");
		alert.setContentText("You have entered invalid date format!\nExpected fromat is \"" + Constants.DATE_TIME_PATTERN + "\" or \"" +
								Constants.IF_DATE_IS_NULL + "\" for unknown date.");
		alert.showAndWait();
	}
	
	private String getSelectedTextFromWebviewAndSetcustomFromTextField() {
    	if (lastSelectedVertex != null) {
    		String selection = (String) webView.getEngine().executeScript("window.getSelection().toString()");
    		customFromTextField.setText(lastSelectedVertex.getName());
    		return selection;
    	}
    	return "";
	}
	
//	private void invokeInfoboxPicker() {
//		FXMLLoader loader = new FXMLLoader(getClass().getResource(INFOBOX_PICKER_WINDOW_XML_FILE));
//		InfoboxPickerController infoboxPickerController = new InfoboxPickerController(this, jgraphtDataHandler, graphVisualized);
//		loader.setController(infoboxPickerController);
//						
//		Pane rootPane = null;
//		
//		try {
//			rootPane = (Pane) loader.load();			
//		} catch (Exception e) {
//			e.printStackTrace();
//			Alert alert = new Alert(AlertType.ERROR);
//			alert.setTitle("Window cannot be created");
//			alert.setContentText("file: " + INFOBOX_PICKER_WINDOW_XML_FILE + "\n" + 
//					             "Error: " + e.getMessage() + "\n" +
//					             "Exception: " + e.getClass().getName());			
//			alert.showAndWait();
//			System.exit(1);
//		}
//        Stage stage = new Stage();
//        stage.setTitle("Infobox picker");
//        stage.setScene(new Scene(rootPane, 450, 450));
//        stage.showAndWait();
//	}
	
	public void fillGraph(String startVertex, String linkingProperty) throws SQLException {
		List<String> addedVertices = new ArrayList<String>();
		Queue<String> queue = new ArrayDeque<String>();
		queue.add(startVertex);
		addedVertices.add(startVertex);
		try {
			jgraphtDataHandler.insertInfoboxIntoGraph(startVertex);
		}  catch (SQLException e) {
			Alert alert = new Alert(AlertType.ERROR);
			alert.setTitle("Database connection error");
			alert.setContentText("There is problem with connection to the database. Check if database is reachable and try it again after restarting program.");			
			alert.showAndWait();
		}
		
		String currentVertex;
		
		String keyProperty = "";
		if (linkingProperty.contains("[") && linkingProperty.contains("]")) {
			keyProperty = linkingProperty.substring(linkingProperty.indexOf("[") + 1);
			keyProperty = keyProperty.substring(0, keyProperty.indexOf("]"));
			
			linkingProperty = linkingProperty.substring(0, linkingProperty.indexOf("[") - 1);
		}
	    
	    while (!queue.isEmpty()) {
	        currentVertex = queue.remove();
	        
	        InfoboxEntry currentInfobox = databaseHandler.getInfobox(currentVertex);
	        
	        if (currentInfobox == null) {
	        	continue;
	        }
	        
	        List<KeyWithProperty> infoboxProperties = currentInfobox.getKeys();
	        for (int i = 0; i < infoboxProperties.size(); i++) {
	        	KeyWithProperty infoboxProperty = infoboxProperties.get(i);
//		        if (infoboxProperty == null) {
//		        	continue;
//		        }
	        	if (infoboxProperty.getKey().equals(linkingProperty) && infoboxProperty.getProperty().equals(keyProperty)) {
	        		List<InfoboxLink> linksWithProperty = currentInfobox.getLinks().get(i);
	        		for (InfoboxLink linkWithProperty: linksWithProperty) {
	        			InfoboxEntry newInfoboxFound;
	        			if (linkWithProperty.getLinkValue() != null) {
	        				newInfoboxFound = databaseHandler.getInfobox(linkWithProperty.getLinkValue());	        				
	        			}
	        			else {
	        				newInfoboxFound = databaseHandler.getInfobox(linkWithProperty.getTitleValue());
	        			}
	        			String newVertex;
	        	        if (newInfoboxFound == null) {
	        	        	newVertex = linkWithProperty.getTitleValue();
	        	        }
	        	        else {
	        	        	newVertex = newInfoboxFound.getName();	  
	        	        }
	        	        
//	        			System.out.println(newVertex);
	        			
		        		jgraphtDataHandler.insertInfoboxIntoGraph(newVertex);


	        			GraphVertex currentGraphVertex = jgraphtDataHandler.getVertexByName(currentVertex);
	        			GraphVertex newGraphVertex = jgraphtDataHandler.getVertexByName(newVertex);
	        			
	        			
	        			if (keyProperty.length() > 0) {
	        				jgraphtDataHandler.insertEdge(currentGraphVertex, newGraphVertex, linkingProperty + "[" + currentVertex + "-" + newVertex + "]" 
        							+ " [" + keyProperty + "]");	        				
	        			}
	        			else {
	        				jgraphtDataHandler.insertEdge(currentGraphVertex, newGraphVertex, linkingProperty + "[" + currentVertex + "-" + newVertex + "]");
	        			}

		        		if (!addedVertices.contains(newVertex)) {
		        			queue.add(newVertex);
		        			addedVertices.add(newVertex);
		        		}
	        		}
	        	}
	        }
	    }
	    
	    updateEdgesListView();
//	    graphViewUpdate();
	    updateTableView();
	}
	
	private void updateTableView() {
		tableOverview.getItems().clear();
		for (GraphVertex graphVertex: jgraphtDataHandler.getAllVertices()) {
			TableRowInfoboxData tableRowInfoboxData = new TableRowInfoboxData();
			tableRowInfoboxData.setBirthDateColumn(graphVertex.getBirthDate());
			tableRowInfoboxData.setDeathDateColumn(graphVertex.getDeathDate());
			tableRowInfoboxData.setNameColumn(graphVertex.getName());

			StringBuilder realitonsWithSB = new StringBuilder();
			StringBuilder realitonsNamesSB = new StringBuilder();
			for(RelationshipEdge edgeFromVertex: jgraphtDataHandler.getEdgesOfVertex(graphVertex.getName())) {
				GraphVertex sourceVertex = jgraphtDataHandler.getEdgeSource(edgeFromVertex);
				GraphVertex targetVertex = jgraphtDataHandler.getEdgeTarget(edgeFromVertex);
				if (sourceVertex.equals(graphVertex)) {
					realitonsWithSB.append(targetVertex.getName() + " | ");
				}
				else {
					continue;
				}
				realitonsNamesSB.append(edgeFromVertex.getLabel().substring(0, edgeFromVertex.getLabel().indexOf("[")) + " | ");
			}
			
			if (realitonsWithSB.length() > 3) {
				realitonsWithSB.setLength(realitonsWithSB.length() - 3);				
			}
			if (realitonsNamesSB.length() > 3) {
				realitonsNamesSB.setLength(realitonsNamesSB.length() - 3);
			}
			
			tableRowInfoboxData.setRealitonsWithColumn(realitonsWithSB.toString());
			tableRowInfoboxData.setRelationsColumn(realitonsNamesSB.toString());
			
			tableOverview.getItems().add(tableRowInfoboxData);
		}

		tableOverview.sort();
		
		updateTableSelection();
	}
	
	private void updateTableSelection() {
		if (lastSelectedVertex != null) {
			for (TableRowInfoboxData tableRow: tableOverview.getItems()) {
				if (tableRow.getNameColumn().equals(lastSelectedVertex.getName())) {
					tableOverview.getSelectionModel().select(tableRow);
					break;
				}
			}
		}
	}

	private void updateEdgesListView() {
		edgesListView.getItems().clear();
		for (RelationshipEdge edge: jgraphtDataHandler.getAllEdges()) {
			edgesListView.getItems().add(edge.getLabel());
		}
	}
	
	private void updateWebView() {
//		webView.getEngine().loadContent("https://en.wikipedia.org/wiki/" + lastSelectedVertex.getName().replace(" ", "_"));
		if (lastSelectedVertex != null) { 
			webView.getEngine().load("https://en.m.wikipedia.org/wiki/" + lastSelectedVertex.getName().replace(" ", "_"));
		}
	}

//	private void createNewEdge(Vertex<String> lastDoubleClickedVertex, Vertex<String> vertex) {
//		FXMLLoader loader = new FXMLLoader(getClass().getResource(EDGE_CREATOR_XML_FILE));
//		EdgeCreatorController edgeCreatorController = new EdgeCreatorController(this, lastDoubleClickedVertex, vertex, graphVisualized);
//		loader.setController(edgeCreatorController);
//						
//		Pane rootPane = null;
//		
//		try {
//			rootPane = (Pane) loader.load();			
//		} catch (Exception e) {
//			e.printStackTrace();
//			Alert alert = new Alert(AlertType.ERROR);
//			alert.setTitle("Window cannot be created");
//			alert.setContentText("file: " + EDGE_CREATOR_XML_FILE + "\n" + 
//					             "Error: " + e.getMessage() + "\n" +
//					             "Exception: " + e.getClass().getName());			
//			alert.showAndWait();
//			System.exit(1);
//		}
//        Stage stage = new Stage();
//        stage.setTitle("Edge creator");
//        stage.setScene(new Scene(rootPane, 450, 450));
//        stage.showAndWait();
//	}
		
	public ListView<String> getNodesListView() {
		return edgesListView;
	}
	
}
