package main;

import java.sql.SQLException;

import fxmlcontrollers.MainWindowController;
import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;

public class Main extends Application {
	
	private static final String MAIN_WINDOW_XML_FILE = "/fxml/mainWindow.fxml";

    public static void main(String[] args) {
    	launch(args);
    }
    
	@Override
	public void start(Stage primaryStage) {
		primaryStage.setScene(createScene());
		primaryStage.setMaximized(true);
		primaryStage.setTitle("Wikipedia graph data miner");
		primaryStage.show();
	}
	
	private Scene createScene() {
		FXMLLoader loader = new FXMLLoader(getClass().getResource(MAIN_WINDOW_XML_FILE));
		MainWindowController mainWindowController = null;
		try {
			mainWindowController = new MainWindowController();			
		} catch (SQLException e) {
			Alert alert = new Alert(AlertType.ERROR);
			alert.setTitle("Database connection error");
			alert.setContentText("There is problem with connection to the database. Check your database configuration and try it again.");			
			alert.showAndWait();
			System.exit(1);
		}
		loader.setController(mainWindowController);
						
		Pane rootPane = null;
		
		try {
			rootPane = (Pane) loader.load();			
		} catch (Exception e) {
			e.printStackTrace();
			Alert alert = new Alert(AlertType.ERROR);
			alert.setTitle("Window cannot be created");
			alert.setContentText("file: " + MAIN_WINDOW_XML_FILE + "\n" + 
					             "Error: " + e.getMessage() + "\n" +
					             "Exception: " + e.getClass().getName());			
			alert.showAndWait();
			System.exit(1);
			return null;
		}

		Scene mainScene = new Scene(rootPane);
		
		return mainScene;
	}
}
